@@ -203,11 +203,16 @@ frdp_channel_clipboard_finalize (GObject *object)
   FrdpChannelClipboard        *self = (FrdpChannelClipboard *) object;
   FrdpChannelClipboardPrivate *priv = frdp_channel_clipboard_get_instance_private (self);
 
+  g_signal_handler_disconnect (priv->gtk_clipboard,
+                               priv->clipboard_owner_changed_id);
+
   g_hash_table_unref (priv->remote_files_requests);
   fuse_session_unmount (priv->fuse_session);
   fuse_session_exit (priv->fuse_session);
 
-  _gtk_clipboard_clear_func (priv->gtk_clipboard, self);
+  if (priv->remote_data_in_clipboard)
+    gtk_clipboard_clear (priv->gtk_clipboard);
+
   g_clear_pointer (&priv->fuse_directory, g_free);
 
   g_mutex_lock (&priv->lock_mutex);
@@ -217,6 +222,7 @@ frdp_channel_clipboard_finalize (GObject *object)
 
   g_mutex_unlock (&priv->lock_mutex);
 
+  g_thread_join (priv->fuse_session_thread);
   g_mutex_clear (&priv->fuse_mutex);
   g_mutex_clear (&priv->lock_mutex);
 
