@@ -240,6 +240,11 @@ frdp_display_disconnected (GObject  *source_object,
                            gpointer  user_data)
 {
   FrdpDisplay *self = FRDP_DISPLAY (user_data);
+  FrdpDisplayPrivate *priv = frdp_display_get_instance_private (self);
+
+  g_signal_handlers_disconnect_by_func (priv->session, G_CALLBACK (frdp_display_error), self);
+  g_signal_handlers_disconnect_by_func (priv->session, G_CALLBACK (frdp_display_disconnected), self);
+  g_signal_handlers_disconnect_by_func (priv->session, G_CALLBACK (frdp_display_auth_failure), self);
 
   g_signal_emit (self, signals[RDP_DISCONNECTED], 0);
 
@@ -640,14 +645,14 @@ frdp_display_authenticate (FrdpDisplay  *self,
   FrdpDisplayPrivate *priv = frdp_display_get_instance_private (self);
   GMainContext       *context;
 
-  g_signal_emit (self, signals[RDP_NEEDS_AUTHENTICATION], 0);
-
   priv->awaiting_authentication = TRUE;
 
+  g_signal_emit (self, signals[RDP_NEEDS_AUTHENTICATION], 0);
+
   context = g_main_context_default ();
 
   while (priv->awaiting_authentication)
-    g_main_context_iteration (context, FALSE);
+    g_main_context_iteration (context, TRUE);
 
   *username = *password = *domain = NULL;
   g_object_get (priv->session,
@@ -723,7 +728,7 @@ frdp_display_certificate_verify_ex (FrdpDisplay *display,
   context = g_main_context_default ();
 
   while (priv->awaiting_certificate_verification)
-    g_main_context_iteration (context, FALSE);
+    g_main_context_iteration (context, TRUE);
 
   return priv->certificate_verification_value;
 }
@@ -761,7 +766,7 @@ frdp_display_certificate_change_verify_ex (FrdpDisplay *display,
   context = g_main_context_default ();
 
   while (priv->awaiting_certificate_change_verification)
-    g_main_context_iteration (context, FALSE);
+    g_main_context_iteration (context, TRUE);
 
   return priv->certificate_change_verification_value;
 }
